#include <stdio.h>
#include "contrl.h"

/**********************************************************************

	PED module SYS1 configured for the rxvt terminal used by 
	YAZE-AG.

	Recommended colour selections:

		Normal video	YELLOW
		Inverse video	CYAN on BLUE
		Highlight		WHITE

**********************************************************************/

/*
** Output a single character to the screen using BDOS 6 - Direct
*/

void	outchar(char a)
{
    bdos(6,a);
}

/*--------------------------------------------------------------------*/

void	vdu_xy(unsigned short x, unsigned short y);

/*--------------------------------------------------------------------*/

/*
** Output a string to the screen using outchar so that ^S will not stop
**	the display
*/

void outstr(char *str)
{
    while (*str)
        outchar(*str++);
}

/*--------------------------------------------------------------------*/

void video(mode)
short int mode;
{
    outstr("\33[0;");
    switch (mode)
    {
    case REVERSE:	/* screen reverse video on */
        outstr("47;1;36");
        break;

	case NORMAL:	/* Standard video */
		outstr("1;33");
		break;

    case DIM:		/* Set alternative highlighting mode for
					   block operations */
        outstr("1;40;33");
    }
    outchar('m');
}

/*--------------------------------------------------------------------*/

/*
**	clear to end of line
*/

void	clreol()

{
    outstr("\33[K");
}

/*--------------------------------------------------------------------*/

/*
**	clear to end of screen 
*/

void	clreos()

{
    outstr("\33[J");
}

/*--------------------------------------------------------------------*/

/*
**	clear screen 
*/

void	clrscr()
{
    /* Ordinarily one could simply send <esc>[2J but rxvt doesn't
       always work and does a clear to end-of-screen instead of
       clearing the entire screen.  This is safe.  It works for
       rxvt and for any ANSI terminal. */
    vdu_xy(1,1);
    clreos();
}


/*---------------------------------------------------------------*/
del_line()

/*
**	screen function to delete the cursor line
**	 - causes upward scrolling of the screen
*/
{
    outstr("\33[M");
}

/*---------------------------------------------------------------*/
ins_line()

/*
**	screen function to insert a line above the cursor line
**	 - causes downward scrolling of the screen
*/
{
    outstr("\33[L");
}

/*------------------------------------------------------------*/

/*
** to position the cursor on the vdu at location (x,y)
**		x = line number  [ 1 to scr_length ]
**		y = col  number  [ 1 to lne_length ]
*/

void	vdu_xy(unsigned short x, unsigned short y)
{
    char pos[12];
    sprintf(pos, "\33[%d;%dH", x, y);
    outstr(pos);
}

/*--------------------------------------------------------------------*/

/*
**	FIFO buffer keyboard read routine
**
**		by specifying the operation type a number of things can
**		be done :
**		 type		operation
**		-------------------------------
**		   0 - read next character from buffer with a pre-read
**			of the keyboard.
**		   1 - read character from keyboard and put in buffer 
**			if not NULL.
**		   2 - initialise buffer zeroing it and setting up pointers.
**		   3 - return next character without removing it.
*/

char inchar(type)

char	type;

{
#define BUFFSIZE 20

static char *buf_sta,*buf_end,*buf_inp,*buf_out;
static char buffer[BUFFSIZE];

/* values of type : */
#define OUTPUT	0
#define	INPUT	1
#define INIT	2
#define VIEW	3

/* on OUTPUT and VIEW a keyboard read is done first into the buffer */

    char	ch,i;

    ch = 0;
    if (type==INPUT || type==OUTPUT || type==VIEW)
    {
	if ((buf_inp==buf_end ? buf_sta : buf_inp+1)!=buf_out)
	{
	    i=1;
	    do
		*buf_inp = (char *)(bdos(6,0xFF));
	    while (!*buf_inp && --i);
		if(*buf_inp)
		    buf_inp=buf_inp==buf_end?buf_sta:buf_inp+1;
	}
	if ((type==OUTPUT || type==VIEW) && (buf_out != buf_inp))
	{
	    ch = *buf_out;
	    if (type != VIEW)
		buf_out=buf_out==buf_end?buf_sta:buf_out+1;
	}
    }
    else if (type==INIT)
    {
	buf_sta = buffer;
	buf_end = &buffer[BUFFSIZE-1];
	buf_out = buf_inp = buf_sta;
    }
    return ch;
}

/*--------------------------------------------------------------------*/

/*
**	This routine parses a string to determine a CPM breakup 
**	[ d:n.e ] of a filename into :
**
**	DRIVE   	[d]	1 character
**	FILENAME	[n]	8     "
**	EXTENSION	[e]	3     "
**	given
**	STRING		[name]
**
**	The routine is called with the following
**
**	file_name(name,d,n,e)
**
**	where all passed variables are string names or variable addresses
**
**	The routine returns 0 if an error occurs or 1 if ok.
**	If the '.' is specified without [e] then [e] is set to ' '
**	while if '.' is not given e is NULL.
*/

short	file_name(name, d, n, e)

char	*name, *d,*n,*e;

{
	char *index();
	char *i1,*i2,*i3;

	i1=index(name,':');
	i2=index(name,'.');
	i3=name;
	while (*i3)			/* end of string */
		++i3;

	while (*name==' ')		/* remove leading blanks */
		name++;

	if (i1 == 0)
	{				/* no drive specified */
		*d = 0;
		i1=name;		/* start of filename */
	}
	else if (i1>name+1)
	{				/* could be DEVICE */
		strncpy(n, name, i1-name+1);
		return 1;
	}
	else
	{
		*d = *(i1-1);
		*(d+1) = 0;
		i1 += 1;		/* start of file name */
	}

	if (i2 == 0)			/* period not specified */
	{
		*e = 0;			/*  no extension given  */
		if (i3>i1+8)		/* filename too long */
			return 0;
		strcpy( n, i1 );
	}
	else if (i2>i1+9 || i3>i2+4)
		return 0;		/* too long */
	else{
		*i2++ = 0;		/* make name a string + pt to ext */
		strcpy(n,i1);
		strcpy(e,i2);
		if (*e == 0)		/* set to blank for differentiation */
			strcpy(e," ");
	}
	return 1;			/* successful parse */
}

/*---------------------------------------------------------------------*/

/*
**	Construct a filename given the drive,name and extension.
**	The filename is built to CP/M convention. The drive spec
**	can be excluded for default specification.
**	Returns the pointer to the string t.
*/
char	*name(t,d,n,e)

char	*t,*d,*n,*e;

{
	if (index(n,':'))
	{			/* if a possible DEVICE  ie aaa: */
		strcpy(t,n);
		*d = *e = 0;
	}
	else if(*d==NULL)
		sprintf( t,"%s.%s",n,e);
	else
		sprintf( t,"%s:%s.%s",d,n,e);
	return t;
}

/*---------------------------------------------------------------------*/

/*
**
**	Routine to provide Buffered Input from terminal
**	If ESC is pressed input is aborted
**	TAB is replaced with a space
**
*/

buff_input(char *ch)
{

static char	delete[]	={'\10',' ','\10','\0'};

char	i,*tmp;

	for (i=1,tmp=ch; i<80; tmp++,i++)
	{
		do
			*tmp = inchar(0);
		while (!(*tmp));
		if (*tmp==0X1B)
		{
			ch[0]= *tmp;
			break;
		}
		else if (*tmp=='\r')
		{
			*tmp = '\0';	/* always terminate with a NULL */
			break;
		}
		else if (*tmp=='\t')
			*tmp=' ';
		else if ((*tmp==0X7F)||(*tmp==0X08))
		{
			if( i <= 1) break;
			outstr(delete);
			*tmp-- = ' ';
			*tmp-- = ' ';
			i -= 2;
		}
		else
			outchar( *tmp);
	}
	ch[80] = 0;
}

/*---------------------------------------------------------------------*/

/*
**	Routine to search for a string s_str in another string
**	commencing at the pointer txt_ptr
**
*/

char	*strsch( txt_ptr, s_str)

char	*s_str,*txt_ptr;
{
	while (*txt_ptr)
	{
		if (strncmp(txt_ptr,s_str,strlen(s_str)) == 0)
			return txt_ptr;
		txt_ptr++;
	}
	return NULL;
}

/*---------------------------------------------------------------------*/

/*
**	Routine to convert a string to UPPER case given the pointer to
**	the string [ str ]
**		returns the pointer to the string
**
*/

char	*strtoup( str)

char	*str;

{
	char	*s;

	s = str;
	while (*s)
	{
		if (*s>='a' && *s<='z')
	/* recommend limiting the use of toupper so that it is consistent
		with all libraries !! */
			*s=toupper(*s);
		s++;
	}
	return str;
}
